import shutil
import vs
import os
import datetime
import pathlib

# Creates a directory if not present
def ValidateDirectory (strPath):
    if not os.path.isdir(strPath):
        try:
            pathlib.Path(strPath).mkdir(parents=True, exist_ok=True)
        except:
            Ignore = True
            #vs.AlertCritical('Unable to create folder:',strPath)
            return False
    # Path exist
    return True

# Removes a directory
def RemoveDirectory (strPath):
    if os.path.isdir(strPath):
        try:
            os.remove(strPath)
            return True
        except:
            Ignore = True
            #vs.AlertCritical('Unable to remove folder:',strPath)
    return False

# Returns platform specific plug-in file list
def GetFileList ():
    FileList = set()

    major,minor,maintenance,platform = vs.GetVersion()
    if (platform != 1):
        # WINDOWS
        FileList.add('VectorMEP.vwr')
        FileList.add('VectorMEP.vlb')
    else:
        # MAC
        FileList.add('VectorMEP.vwlibrary')
    return FileList
    
    
# If installation went succesfully
VM_RESULT = True

# Create base folders...
VW_USER_LIBARY_PATH = os.path.normpath(vs.GetFolderPath(-13))
VW_USER_LIBARYDefault_PATH = os.path.normpath(vs.GetFolderPath(-120))
VW_USER_WORKSPACE_PATH = os.path.normpath(vs.GetFolderPath(-4))
VW_USER_PLUGINS_PATH = os.path.normpath(vs.GetFolderPath(-2))

# Custom subfolder for better organisation
VM_PLUGINFOLDER = 'VectorMEP'
VM_INSTALLATIONFOLDER = 'VectorMEP Installation Files'

# List of plug-in file shipped with installer
FileList = GetFileList()

# Folders shipped with the plug-in
SOURCE_VMEP_Installation = os.path.join(VW_USER_PLUGINS_PATH,VM_INSTALLATIONFOLDER)
SOURCE_VMEP_PluginFolder = os.path.join(VW_USER_PLUGINS_PATH,VM_INSTALLATIONFOLDER,'Plug-ins')
SOURCE_VMEP_LibraryFolder = os.path.join(VW_USER_PLUGINS_PATH,VM_INSTALLATIONFOLDER,'Libraries')
SOURCE_VMEP_LibraryDefaultsFolder = os.path.join(VW_USER_PLUGINS_PATH,VM_INSTALLATIONFOLDER,'Libraries/Defaults/Reports_Schedules')

# Destination folders for file placement
DEST_VMEP_PluginFolder = os.path.join(VW_USER_PLUGINS_PATH,VM_PLUGINFOLDER)
DEST_VMEP_LibraryFolder = os.path.join(VW_USER_LIBARY_PATH,VM_PLUGINFOLDER)
DEST_VMEP_LibraryDefaultFolder = os.path.join(VW_USER_LIBARYDefault_PATH,VM_PLUGINFOLDER)

# Make sure the VectorMEP plug-in folder Exists
VM_RESULT = ValidateDirectory(DEST_VMEP_PluginFolder)

vs.AlertInformHLink('VectorMEP installer', 'Welcome to the VectorMEP family. Make sure to visit', 'the VectorMEP wiki', 'https://www.vectormep.com/Wiki', 'for more information about our software.', False)

if VM_RESULT:
    # Moves plug-in files into our own VectorMEP folder
    for FileName in FileList:
        SOURCE_FileLocation = os.path.join(SOURCE_VMEP_PluginFolder, FileName)
        DEST_FileLocation = os.path.join(DEST_VMEP_PluginFolder,FileName)
        try:
            if os.path.exists(DEST_FileLocation):#MAC fix for the vwlibrary file ending up as part of an existing package recurv.
                shutil.rmtree(DEST_FileLocation, ignore_errors=True)
            shutil.move(SOURCE_FileLocation,DEST_FileLocation)
        except shutil.Error as err:
            vs.AlertCritical('Unable to move installation file.' + err.args[0],SOURCE_FileLocation + ' => ' + DEST_FileLocation)
            VM_RESULT = False
        
    if VM_RESULT:
        # remove empty folder. 
        RemoveDirectory(SOURCE_VMEP_PluginFolder)

    # Move library
    # Check for existing library
    RefreshLibrary = True
    if os.path.exists(DEST_VMEP_LibraryFolder):
        DLG_RESULT = vs.AlertQuestion('An existing VectorMEP library was detected. How would you like to proceed?', 'Keep: Keep your existing library (advised) \nRefresh: Backup the old folder and reset the library to the defaults', 1, 'Keep', 'Refresh','','')

        RefreshLibrary = DLG_RESULT == 0

        if DLG_RESULT == 0:
            try:
                os.rename(DEST_VMEP_LibraryFolder, DEST_VMEP_LibraryFolder + '_' + datetime.datetime.now().strftime("%Y-%m-%d %H-%M"))
            except:
                vs.AlertCritical('An older VectorMEP library folder was detected but could not be modified','Please refer to the manual installation instructions.')
                VM_RESULT = False

    if RefreshLibrary :
        SOURCE_FromLib = os.path.join(SOURCE_VMEP_LibraryFolder,VM_PLUGINFOLDER)
        try:
            shutil.move(SOURCE_FromLib,DEST_VMEP_LibraryFolder)
        except:
            vs.AlertCritical('Could not move library','Please manually move the folder contents of"' + FromLib + '" into "' + DEST_VMEP_LibraryFolder)
            VM_RESULT = False
    # Move WS scripts folder
        SOURCE_FromLibWSDefaults = os.path.join(SOURCE_VMEP_LibraryDefaultsFolder,VM_PLUGINFOLDER)
        try:
        	# Remove file if present because it has no use to backup the old file.
            if os.path.exists(DEST_VMEP_LibraryDefaultFolder):
                shutil.rmtree(DEST_VMEP_LibraryDefaultFolder)
            shutil.move(SOURCE_FromLibWSDefaults,DEST_VMEP_LibraryDefaultFolder)
        except:
            vs.AlertCritical('Could not move library defaults','Please manually move the folder contents of"' + FromLib + '" into "' + DEST_VMEP_LibraryDefaultFolder)
            VM_RESULT = False

    # remove empty library folder
    try:
        os.remove(SOURCE_VMEP_LibraryFolder)
    except:
        Ignore = True

#Try to clean up installation folder...
if os.path.exists(SOURCE_VMEP_Installation):
    shutil.rmtree(SOURCE_VMEP_Installation, ignore_errors=True)
if not VM_RESULT :
    vs.AlertCritical('The VectorMEP plug-in could not be correctly installed.','Please refer to the manual installation instructions.')

